/*
This is part of the audio CD player library
Copyright (C)1998 Tony Arcieri

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*/

#ifndef _CDAUDIO_H
#define _CDAUDIO_H

/* Used with disc_info */
#define CDAUDIO_PLAYING				0
#define CDAUDIO_PAUSED				1
#define CDAUDIO_COMPLETED			2
#define CDAUDIO_NOSTATUS			3

#define MAX_TRACKS				100
#define MAX_SLOTS				100 /* For CD changers */

#define CURRENT_CDDBREVISION			2

/* CDDB defaults */
#define CDDB_DEFAULT_HOSTNAME			"cddb.cddb.com"
#define CDDB_DEFAULT_PORT			80
#define CDDB_MAX_SERVERS			128
#define MAX_INEXACT_MATCHES			16

/* Current CDDB protocol level supported */
#define CDDB_PROTOCOL_LEVEL "3"

/* CDDB genres */
#define CDDB_UNKNOWN				0
#define CDDB_BLUES				1
#define CDDB_CLASSICAL				2
#define CDDB_COUNTRY				3
#define CDDB_DATA				4
#define CDDB_FOLK				5
#define CDDB_JAZZ				6
#define CDDB_MISC				7
#define CDDB_NEWAGE				8
#define CDDB_REGGAE				9
#define CDDB_ROCK				10
#define CDDB_SOUNDTRACK				11

/* CDDB entry */
struct cddb_entry {
   int entry_present;
   unsigned long entry_id;
   int entry_genre;
};

/* Server retrieval structure */
struct cddb_server {
  char server_name[256];
  int server_port;
  int server_index;
  int server_use_proxy;
  char server_proxy[256];
  int server_proxy_port;
};

/* CDDB hello structure */
struct cddb_hello {
   /* Program */
   char hello_program[256];
   /* Program version */
   char hello_version[256];
};

/* An entry in the query list */
struct query_list_entry {
   int list_genre;
   int list_id;
   char list_title[64];
   char list_artist[64];
};

/* CDDB query structure */
struct cddb_query {
#define NOMATCH					0
#define EXACT					1
#define INEXACT					2
   int query_match;
   int query_matches;
   struct query_list_entry query_list[MAX_INEXACT_MATCHES];
};

/* Used for keeping track of times */
struct disc_timeval {
   int minutes;
   int seconds;
};

/* Track specific information */
struct track_info {
   struct disc_timeval track_length;
   struct disc_timeval track_pos;
   int track_start;
};

/* Disc information such as current track, amount played, etc */
struct disc_info {
   int disc_present;				/* Is disc present? */
   int disc_mode;				/* Current disc mode */
   struct disc_timeval track_time;		/* Current track time */
   struct disc_timeval disc_time;		/* Current disc time */
   struct disc_timeval disc_length;		/* Total disc length */
   int disc_frame;				/* Current frame */
   int disc_track;				/* Current track */
   int disc_totaltracks;			/* Number of tracks on disc */
   struct track_info track[MAX_TRACKS];		/* Track specific information */
};

/* Invisible volume structure */
struct __volume { 
   int left;
   int right;
};

/* Volume structure */
struct disc_volume {
   struct __volume vol_front;
   struct __volume vol_back;
};
   
/* Track database structure */
struct track_data {
   char track_name[256];			/* Track name */
   char track_extended[4096];			/* Extended information */
};

/* Disc database structure */
struct disc_data {
   unsigned long data_id;			/* CD id */
   char data_title[256];			/* Disc title */
   char data_artist[256];			/* We may be able to extract this */
   char data_extended[4096];			/* Extended information */
   int data_genre;				/* Disc genre */
   struct track_data data_track[MAX_TRACKS];	/* Track names */
};

/* Summary of a single disc in the changer */
struct disc_summary {
   int disc_present;				/* Is disc present? */
   struct disc_timeval disc_length;		/* Length of disc */
   int disc_totaltracks;			/* Total tracks */
   unsigned long disc_id;			/* CDDB ID */
   int disc_info[128];				/* Artist name / Disc name */
};

/* Disc changer structure */
struct disc_changer {
   int changer_slots;
   struct disc_summary changer_disc[MAX_SLOTS];
};

/* CDDB server list structure */
struct cddb_hostelement {
   char host_hostname[256];
   char host_city[256];
   int host_port;
   int host_icbm_latitude;
   int host_icbm_longitude;
};

/* CDDB server list structure */
struct cddb_serverlist {
   int list_len;
   struct cddb_hostelement list_host[256];
};

/* Function prototopes */
void cd_version(char *buffer, int len);
int cd_init_device(char *device_name);
int cd_stat(int cd_desc, struct disc_info *disc);
int cd_play_frames(int cd_desc, int startframe, int endframe);
int cd_play_track_pos(int cd_desc, int starttrack, int endtrack, int startpos);
int cd_play_track(int cd_desc, int starttrack, int endtrack);
int cd_play_pos(int cd_desc, int track, int startpos);
int cd_play(int cd_desc, int track);
int cd_track_advance(int cd_desc, int endtrack, struct disc_timeval *time);
int cd_advance(int cd_desc, struct disc_timeval *time);
int cd_stop(int cd_desc);
int cd_pause(int cd_desc);
int cd_resume(int cd_desc);
int cd_eject(int cd_desc);
int cd_close(int cd_desc);
int cd_set_volume(int cd_desc, struct disc_volume *vol);
int cd_get_volume(int cd_desc, struct disc_volume *vol);
unsigned long cddb_discid(int cd_desc);
char *cddb_genre(int genre);
static int cddb_connect_server(struct cddb_server *server);
int cddb_get_server(struct cddb_server *server);
int cddb_query(int cd_desc,struct cddb_server *server,struct cddb_hello *hello,
			   struct cddb_query *query);
int cddb_read(int cd_desc,struct cddb_server *server,struct cddb_hello *hello,
			  struct cddb_entry *entry,struct disc_data *data);
int cddb_quit(int sock);
int cddb_read_data(int desc, struct disc_data *data);
int cddb_generate_new_entry(int cd_desc, struct disc_data *data);
int cddb_read_disc_data(int cd_desc, struct disc_data *data);
int cddb_write_disc_data(int cd_desc, struct disc_data *data);
int cddb_erase_entry(struct disc_data *data);
int cddb_stat_disc_data(int cd_desc, struct cddb_entry *entry);
int cd_changer_select_disc(int cd_desc, int disc);
int cd_changer_slots(int cd_desc);
int cd_changer_stat(int cd_desc, struct disc_changer *changer);

#endif
